#!/usr/bin/env python3
# © Copyright 2020-2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

"""
Python script to fetch NSS data for OES telemetry.
"""

import glob
import json
import os
import re
import subprocess
import sys
import time
from collections import OrderedDict

"""
service - Name of the service
properties - Systemctl properties
result - Final result format of NSS telemetry data.
telemetry_result_loc - Location to place NSS data for telemetry
telemetry_config - Telemetry config file's location
"""

service = "novell-nss"
properties = ["ActiveState"]
result = OrderedDict([
    ("OESService", "OES Storage Services (NSS)"),
    ("timestamp", int(time.time()))])
pseudo_name_map = dict()
telemetry_result_loc = "/var/opt/novell/telemetry/data/"
telemetry_config = "/etc/opt/novell/telemetry/config/telemetry.json"


def get_systemctl_properties():
    """
    Function to run systemctl and capture the output.
    :return: dict: Result of systemctl
    """

    systemctl_result = dict()
    cmd = "systemctl show " + service + " --property=" + ",".join(properties)
    output = run_command(cmd)

    for line in output.splitlines():
        key, value = line.strip().split("=")
        systemctl_result[key] = value

    return systemctl_result


def run_command(cmd):
    """
    Function to execute linux command and capture the result
    :param cmd: linux command
    :return: Output of the command
    """

    err, output = None, None
    ret = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    try:
        (output, err) = ret.communicate(timeout=5)
    except TimeoutError:
        ret.kill()

    if err:
        sys.exit(2)
    elif output:
        return output.decode('utf-8')


def parse_nlvm_output(output, pseudo_name):
    """
    Function to parse the nlvm command output and store the results as a dictionary
    :param output: nlvm command output
    :param pseudo_name: name to be used in place of the actual key
    :return: dictionary containing the attributes and values present in the output
    :return: count of the keys being added to the result dictionary
    """

    output_dict = {}
    count = line_num = 1

    output_list = output.splitlines()
    attributes_list = output_list[0].split()

    while line_num < len(output_list):
        values_list = output_list[line_num].split()
        attribute_index = 1
        resource_info_dict = {}
        line_num += 1
        while attribute_index < len(values_list):
            resource_info_dict[attributes_list[attribute_index]] = values_list[attribute_index];
            attribute_index += 1
        if "RAID" in resource_info_dict and resource_info_dict["RAID"] != "No":
            add_raid_list(resource_info_dict)
        else:
            pseudo_name_map[values_list[0]] = pseudo_name + str(count)
            output_dict[values_list[0]] = resource_info_dict
            count += 1

    return output_dict, count - 1


def add_raid_list(resource_info_dict):
    """
    Function to add the RAID device configuration to the result
    :param resource_info_dict: dictionary with information about the RAID device configuration
    """

    if "RAID" not in result:
        result["RAID"] = {"Number of devices": 0}
    result["RAID"]["Number of devices"] += 1
    result["RAID"]["RAID Controller " + str(result["RAID"]["Number of devices"])] = resource_info_dict


def add_snaps_list():
    """
    Function to add NSS snapshot configuration to the result.
    """

    cmd = "/sbin/nlvm list snaps more"
    output = run_command(cmd)

    if output:
        if output.strip() == "No snapshots":
            result["Snapshots"]  = "No snapshots"
        else:
            snap_dict = OrderedDict([
                ("Number of snapshots", "")])
            snap_dict, count = parse_nlvm_output(output, "SNAPSHOT")
            snap_dict["Number of snapshots"] = count
            result["Snapshots"] = snap_dict


def add_device_list():
    """
    Function to add device information to the result.
    """

    cmd = "/sbin/nlvm list devices"
    output = run_command(cmd)

    if output:
        device_dict = OrderedDict([
                ("Number of devices", "")])
        device_dict, count = parse_nlvm_output(output, "DISK")
        device_dict["Number of devices"] = count
        result["Devices"] = device_dict


def add_pool_list():
    """
    Function to add all NSS pools and their attributes to the result.
    """

    cmd = "/sbin/nlvm list pools more"
    output = run_command(cmd)

    if output:
        pool_dict = OrderedDict([
            ("Number of pools", "")])
        pool_dict, count = parse_nlvm_output(output, "POOL")
        pool_dict["Number of pools"] = count
        result["Pools"] = pool_dict


def add_volume_list():
    """
    Function to add all NSS volumes and their attributes to the result.
    """

    cmd = "/sbin/nlvm list volumes more"
    output = run_command(cmd)

    if output:
        volume_dict = OrderedDict([
            ("Number of volumes", "")])
        volume_dict, count = parse_nlvm_output(output, "VOLUME")
        volume_dict["Number of volumes"] = count
        result["Volumes"] = volume_dict


def add_media_list():
    """
    Function to add media information for the NSS pools to result
    """

    cmd = "/sbin/nss /PoolMediaVersion"
    output = run_command(cmd)

    if output:
        media_info = output.splitlines()[2 : -1]
        for line in media_info:
            pool_name = line.split()[0]
            result["Pools"][pool_name]["Media Version"] = line.split()[1]
            result["Pools"][pool_name]["Media Name"] = line.split()[3]


def add_trustee_list():
    """
    Function to add the trustees count for each of NSS volumes to result
    """
    volume_list = list(result["Volumes"].keys())
    volume_list.remove("Number of volumes")
    trustee_count = {}
    for volume in volume_list:
       if volume not in trustee_count:
            trustee_count[volume] = {"eDir users": "NA", "AD users": "NA"}
       cmd = "grep -wc \"<trustee>\" /_admin/Manage_NSS/Volume/" + volume + "/TrusteeInfo.xml"
       count = run_command(cmd)

       if count:
           trustee_count[volume]["eDir users"] = int(count.rstrip())

       cmd = "grep -wc \"<trustee type=\\\"adID\\\">\" /_admin/Manage_NSS/Volume/" + volume + "/TrusteeInfo.xml"
       count = run_command(cmd)

       if count:
           trustee_count[volume]["AD users"] = int(count.rstrip())

    result["Trustee count"] = trustee_count


def add_connection_list():
    """
    Function to get the total count of connections made to NSS
    """

    cmd = "/sbin/nss /listconn"
    output = run_command(cmd)
    if output:
        result["Number of CIFS connections"] = 0
        connection_info = output.splitlines()
        for line in connection_info:
            if "refCount[0]:" in line:
                count = int(line.split()[1].rstrip())
                result["Number of CIFS connections"] += count


def add_error(msg):
    """
    Function to add error to result
    :param msg: Error message
    """

    if "Error" in result:
        result["Error"] += [msg]
    else:
        result["Error"] = [msg]


def write_result():
    """
    Function to remove old, create new and update the file with result.
    """

    files = glob.glob(telemetry_result_loc + 'NSS_*.json')
    for file in files:
        try:
            os.remove(file)
        except OSError as e:
            add_error("Unable to delete old data file: " + e.strerror)

    file_name = "NSS_" + str(result["timestamp"]) + ".json"
    file_path = telemetry_result_loc + file_name
    file = open(file_path, "w")
    json_string = json.dumps(result)
    for key in pseudo_name_map:
        json_string = re.sub(r"\b" + re.escape(key) + r"\b", pseudo_name_map[key], json_string)
    file.write(json_string)
    file.close()


def telemetry_check():
    """
    To check if telemetry is enabled.
    :return: Boolean
    """

    try:
        with open(telemetry_config, "r") as file:
            status = json.load(file)
            if status["opt-in"].lower() == "yes":
                return True
    except IOError:
        pass

    return False


def generate_nss_data():
    """
    The main function to genereate the data of NSS for Telemetry.

    :return 0 on success
    """

    # generate only if telemetry is enabled
    enabled = telemetry_check()
    if enabled:
        service_properties = get_systemctl_properties()
        if service_properties != -1:
            state = service_properties.get("ActiveState")
            if state == "active":
                add_device_list()
                add_pool_list()
                add_media_list()
                add_snaps_list()
                add_volume_list()
                add_trustee_list()
                add_connection_list()
            else:
                add_error("nss is down")
        write_result()

    return 0


# Script starts from here
if __name__ == '__main__':
    generate_nss_data()
