#!/usr/bin/env perl

# © Copyright 2020 Micro Focus or one of its affiliates.
#Copyright © [2007] Novell, Inc.  All Rights Reserved.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

########################################################################
# Perl script to emulate NetWare's volmn command.
########################################################################

{
    my $rc;
    my $i;
    my $comment;

    if ($#ARGV < 0 || $ARGV[0] eq "/?" || $ARGV[0] eq "-?")
    {
        DisplayHelp();
        exit;
    }

    if ($ARGV[0] eq "help")
    {
        $rc = DisplayHelp();
        exit $rc;
    }

    open(NSSFILE, "+</_admin/Manage_NSS/manage.cmd")
        or die "Error opening NSS management file ($!) on server";

    my $command = $ARGV[0];
    if ($command eq "cancel")
    {
	if ($#ARGV ne 1)
	{
	    die "Usage: volmn cancel <opID>\n";
	}
	$rc = Cancel($ARGV[1]);
    }
    elsif ($command eq "list")
    {
	if ($#ARGV ne 1)
	{
	    die "Usage: volmn list <opID>\n";
	}
	$rc = List($ARGV[1]);
    }
    elsif ($command eq "move")
    {
	if ($#ARGV ne 3)
	{
	    die "Usage: volmn move <srcVol> <dstVol> <dstServer>\n";
	}
	$rc = Move($ARGV[1],$ARGV[2],$ARGV[3]);
    }
    elsif ($command eq "pause")
    {
	if ($#ARGV eq 0)
	{
	    die "Usage: volmn pause <opID> [<comment>]\n";
	}
	if ($#ARGV >= 2)
	{
	    $comment = $ARGV[2];
	    for ($i = 3; $i <= $#ARGV; $i++)
	    {
		$comment = $comment . " " . $ARGV[$i];
	    }
	}
	else
	{
	    $comment = "";
	}
	$rc = Pause($ARGV[1],$comment);
    }
    elsif ($command eq "resume")
    {
	if ($#ARGV ne 1)
	{
	    die "Usage: volmn resume <opID>\n";
	}
	$rc = Resume($ARGV[1]);
    }
    elsif ($command eq "split")
    {
	if ($#ARGV ne 3)
	{
	    die "Usage: volmn split <srcPath> <dstPath> <dstServer>\n";
	}
	$rc = Split($ARGV[1],$ARGV[2],$ARGV[3]);
    }
    elsif ($command eq "status")
    {
	if ($#ARGV ne 0)
	{
	    die "Usage: volmn status\n";
	}
	$rc = Status();
    }
    else
    {
        print "Unknown option: $ARGV[0]\n".
            "type volmn help for help.\n";
        $rc = 1;
    }

    close(NSSFILE);
    exit $rc;
}


########################################################################
# Process volmn help command
########################################################################
sub DisplayHelp()
{
    print "volmn commands:\n".
          "  volmn cancel <opID>: Terminate an operation\n".
          "  volmn help:          Display this help message\n".
          "  volmn list <opID>:   Display skipped files for an operation\n".
          "  volmn move <srcVol> <dstVol> <dstServer>:  Start a new move operation running\n".
          "  volmn pause <opID> <comment>:  Pause a running operation\n".
          "  volmn resume <opID>: Resume a paused operation\n".
          "  volmn split <srcPath> <dstPath> <dstServer>: Start a new split operation running\n".
          "  volmn status:        Display the current status of all known operations\n".
          "\n";

    return 0;
}

########################################################################
# Process volmn cancel command
########################################################################
sub Cancel($)
{
    my $ret;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><volMn>".
				       "<changeJobState>".
				         "<id>".$_[0]."</id>".
				         "<abort/>".
				       "</changeJobState>".
                                     "</volMn></nssRequest>");
#	print "result= $nssResult\n";
	$ret = GetResultCode($nssResult, "changeJobState");
	if ($ret == -10020)
	{
	    print "No such process: $_[0]\n";
	}
	else
	{
	    $ret = ParseBasicReply($nssResult, "changeJobState");
	}
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }

    return $ret;    
}

########################################################################
# Process volmn list command
########################################################################
sub List($)
{
    my $ret;
    my $cookie = 0;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
	do
	{
	    my $nssResult = WriteCommand(NSSFILE, 
					 "<nssRequest><volMn>".
					 "<listSkippedFiles>".
				           "<id>".$_[0]."</id>".
				           "<cookie>".$cookie."</cookie>".
					 "</listSkippedFiles>".
					 "</volMn></nssRequest>");
	    $ret = GetResultCode($nssResult, "listSkippedFiles");
	    if ($ret == -10020)
	    {
		print "No such process: $_[0]\n";
	    }
	    else
	    {
		$ret = ParseSkippedFileReply($nssResult);
	    }
	    if ($ret == 0)
	    {
		$cookie = GetCookie($nssResult);
	    }
	    else
	    {
		return $ret;
	    }
	} until ($cookie eq "DONE") || ($cookie eq "");
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }

    return $ret;    
}

########################################################################
# Process volmn move command
########################################################################
sub Move($$$)
{
    my $srcVol = $_[0];
    my $dstVol = $_[1];
    my $dstServer = $_[2];
    my $uname;
    my $pwd;
    my $ret;
    my $tree;
    my $treeInfo;

    $uname = GetUserName();
    $pwd   = GetPassword();

    # Remove trailing colon from volume names if supplied
    $srcVol =~ s/:$//;
    $dstVol =~ s/:$//;

    # Check if tree name was provided
    if (substr($dstServer, -1) eq ".")
    {
	chop($dstServer);
	$tree = substr($dstServer, rindex($dstServer, ".") + 1);
	$dstServer = substr($dstServer, 0, rindex($dstServer, "."));
    }
    if ($tree ne "")
    {
	$treeInfo = "<tgtTree>$tree</tgtTree>";
    }
    else
    {
	$treeInfo = "";
    }

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><volMn>".
				       "<createJob>".
				         "<srcVol>$srcVol</srcVol>".
				         "<tgtServer>$dstServer</tgtServer>".
				         $treeInfo.
				         "<tgtVol>$dstVol</tgtVol>".
				         "<time>NOW</time>".
				         "<user>$uname</user>".
				         "<password>$pwd</password>".
				       "</createJob>".
                                     "</volMn></nssRequest>");
#	print "result= $nssResult\n";
	$ret = GetResultCode($nssResult, "createJob");
	if ($ret == -10020)
	{
	    print "No such process: $_[0]\n";
	}
	else
	{
	    $ret = ParseBasicReply($nssResult, "createJob");
	}
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }
}

########################################################################
# Process volmn pause command
########################################################################
sub Pause($$)
{
    my $ret;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><volMn>".
				       "<changeJobState>".
				         "<id>".$_[0]."</id>".
				         "<pause>".$_[1]."</pause>".
				       "</changeJobState>".
                                     "</volMn></nssRequest>");
#	print "result= $nssResult\n";
	$ret = GetResultCode($nssResult, "changeJobState");
	if ($ret == -10020)
	{
	    print "No such process: $_[0]\n";
	}
	else
	{
	    $ret = ParseBasicReply($nssResult, "changeJobState");
	}
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }

    return $ret;    
}

########################################################################
# Process volmn resume command
########################################################################
sub Resume($)
{
    my $ret;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><volMn>".
				       "<changeJobState>".
				         "<id>".$_[0]."</id>".
				         "<resume/>".
				       "</changeJobState>".
                                     "</volMn></nssRequest>");
#	print "result= $nssResult\n";
	$ret = GetResultCode($nssResult, "changeJobState");
	if ($ret == -10020)
	{
	    print "No such process: $_[0]\n";
	}
	else
	{
	    $ret = ParseBasicReply($nssResult, "changeJobState");
	}
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }

    return $ret;    
}

########################################################################
# Process volmn split command
########################################################################
sub Split($$$)
{
    my $srcPath = $_[0];
    my $dstPath  = $_[1];
    my $dstServer = $_[2];
    my $uname;
    my $pwd;
    my $srcVol;
    my $dstVol;
    my $tree;
    my $treeInfo;
    my $pathInfo;

    $uname = GetUserName();
    $pwd   = GetPassword();

    $srcVol  = substr($srcPath, 0, index($srcPath, ":"));
    $srcPath = substr($srcPath, index($srcPath, ":") + 1);

    if (index($dstPath, ":") == -1)
    {
    $dstVol  = $dstPath;
    $dstPath = "";
    }
    else
    {
    $dstVol = substr($dstPath, 0, index($dstPath, ":"));
    $dstPath = substr($dstPath, index($dstPath, ":") + 1);
    }

    # Check if tree name was provided
    if (substr($dstServer, -1) eq ".")
    {
	chop($dstServer);
	$tree = substr($dstServer, rindex($dstServer, ".") + 1);
	$dstServer = substr($dstServer, 0, rindex($dstServer, "."));
    }
    if ($tree ne "")
    {
	$treeInfo = "<tgtTree>$tree</tgtTree>";
    }
    else
    {
	$treeInfo = "";
    }

    if ($dstPath ne "")
    {
    $pathInfo = "<path>$dstPath</path>";
    }
    else
    {
    $pathInfo = "";
    }

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><volMn>".
				       "<createJob>".
				         "<srcVol>$srcVol</srcVol>".
				         "<srcPath>$srcPath</srcPath>".
				         "<tgtServer>$dstServer</tgtServer>".
				         $treeInfo.
				         "<tgtVol>$dstVol</tgtVol>".
				         $pathInfo.
				         "<time>NOW</time>".
				         "<user>$uname</user>".
				         "<password>$pwd</password>".
				       "</createJob>".
                                     "</volMn></nssRequest>");
#	print "result= $nssResult\n";
	$ret = GetResultCode($nssResult, "createJob");
	if ($ret == -10020)
	{
	    print "No such process: $_[0]\n";
	}
	else
	{
	    $ret = ParseBasicReply($nssResult, "createJob");
	}
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }
}

########################################################################
# Process volmn status command
########################################################################
sub Status()
{
    my $ret;
    my $nssResult;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        $nssResult = WriteCommand(NSSFILE, 
				  "<nssRequest><volMn><getJobStatus/>".
				  "</volMn></nssRequest>");
# For testing, enable the following statement:
#	$nssResult = GetTestStatusReply();

        $ret = ParseJobStatus($nssResult);
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }

    return $ret;
}

########################################################################
# Parse the result of the Job Status operation
########################################################################
sub ParseJobStatus($)
{
    my $xml = $_[0];
    my $total = 0;
    my $active = 0;
    my $result;
    my $description;
    my $id;
    my $pct;
    my $state;
    my $operation;
    my $path;
    my $stateString;
    my $extraInfo;
    my $job;
    my @jobs;

    print "Volume Manager Status:\n";

    $result .= GetResultCode($xml, "getJobStatus");
    if ($result == 0)
    {
#	print "Result xml=$xml\n";
	print "  OperationID   % State                            Request\n";

	@jobs = $xml =~ /<job>(.*?)<\/job>/gs;
	foreach $job (@jobs)
	{
	    $id        = GetID($job);
	    $pct       = GetPctComplete($job);
	    $state     = GetState($job);
	    $operation = GetOperation($job);
	    $path      = GetPath($job);

	    $stateString = GetPrintableState($state);
	    $extraInfo   = GetExtraInfo($state, $job);

	    my $output = sprintf("  %11s %3d %-32s %1.5s(%1.19s%s)\n",
				 $id, $pct, $stateString, $operation,
				 $path, "");
	    print $output;
	    if ($extraInfo ne "")
	    {
		print "    $extraInfo\n";
	    }

	    $total++;
	    if (($state eq "Running")       || ($state eq "Scanning")     ||
		($state eq "Updating")      || ($state eq "NameSelect")   ||
		($state eq "Renaming")      || ($state eq "CreateJunc")   ||
		($state eq "Cleanup")       || ($state eq "ReplayingLog") ||
		($state eq "RenameLogFile") || ($state eq "NewLogFile")   ||
		($state eq "MoveTrustees")  || ($state eq "NewEFL")       ||
		($state eq "ReplayingEFL") ||
		($state =~ /^Retry/s))
	    {
		$active++;
	    }
	}
    }
    elsif ($result != -10020)
    {
	$description .= GetErrorDescription($xml, "getJobStatus");
	print "Error: $result ($description)\n";
	return 0;
    }

    print "Total Requests: $total,  $active active operations.\n";
    return 0;
}

########################################################################
# Parse the result of a basic reply (just OK/error)
########################################################################
sub ParseBasicReply($$)
{
    my $xml = $_[0];
    my $request = $_[1];
    my $result;
    my $rc;
    my $text;
    my @xml;

    @xml = $xml =~ /<$request>(.*?)<\/$request>/gs;
    foreach $result (@xml)
    {
        if ($result =~ /<result value=\"(-?\d+)\">/s)
        {
            $rc = $1;
            if ($1 != 0)
            {
		$text = GetErrorText($1);
		if ($text ne "")
		{
		    print "Error $1 ($text)\n";
		}
		else
		{
		    print "Error $1\n";
		}
            }
        }
        if ($result =~ /<description>(.*?)<\/description>/s)
        {
            if ($1 ne "success")
            {
                print "$1\n";
            }
        }
    }
    return $rc;
}

########################################################################
# Parse the result of a skipped file list
########################################################################
sub ParseSkippedFileReply($)
{
    my $xml = $_[0];
    my $result;
    my $filename;
    my $rc;
    my @xml;
    my @fileList;

    print "Parsing $xml\n";
    @xml = $xml =~ /<listSkippedFiles>(.*?)<\/listSkippedFiles>/gs;
    foreach $result (@xml)
    {
        if ($result =~ /<result value=\"(-?\d+)\">/s)
        {
            $rc = $1;
            if ($1 == 0)
	    {
		# List all the <fileName> values
		@fileList = $result =~ /<fileName>(.*?)<\/fileName>/gs;
		foreach $filename (@fileList)
		{
		    # Need to handle CDATA here?
		    print $filename."\n";
		}
	    }
	    else
            {
                print "Error $1\n";
            }
        }
        if ($result =~ /<description>(.*?)<\/description>/s)
        {
            if ($1 ne "success")
            {
                print "$1\n";
            }
        }
    }
    return $rc;
}

########################################################################
# Parse the result of a basic reply (just OK/error)
########################################################################
sub GetResultCode($$)
{
    my $xml = $_[0];
    my $request = $_[1];
    my $result;
    my $rc;
    my @xml;

    @xml = $xml =~ /<$request>(.*?)<\/$request>/gs;
    foreach $result (@xml)
    {
        if ($result =~ /<result value=\"(-?\d+)\">/s)
        {
            $rc = $1;
        }
    }
    return $rc;
}

########################################################################
# Parse the result of an error response
########################################################################
sub GetErrorDescription($$)
{
    my $xml = $_[0];
    my $request = $_[1];
    my $result;
    my $rc;
    my @xml;

    @xml = $xml =~ /<$request>(.*?)<\/$request>/gs;
    foreach $result (@xml)
    {
        if ($result =~ /<description>(.*?)<\/description>/s)
        {
            $rc = $1;
	    return $rc;
        }
    }
    return $rc;
}

########################################################################
# Parse the result of a skipped file list reply
########################################################################
sub GetCookie($)
{
    my $xml = $_[0];
    my $result;
    my $rc;
    my @xml;

    @xml = $xml =~ /<listSkippedFiles>(.*?)<\/listSkippedFiles>/gs;
    foreach $result (@xml)
    {
        if ($result =~ /<cookie>(.*?)<\/cookie>/s)
        {
            $rc = $1;
        }
    }
    return $rc;
}

########################################################################
# Get the operation ID from a Job Status reply
########################################################################
sub GetID($)
{
    my $result = $_[0];
    my $rc;

    if ($result =~ /<id>(\d+)<\/id>/s)
    {
	$rc = $1;
    }
    return $rc;
}

########################################################################
# Get the estimated percent complete from a Job Status reply
########################################################################
sub GetPctComplete($)
{
    my $result = $_[0];
    my $rc;

    if ($result =~ /<pctComplete>(\d+)<\/pctComplete>/s)
    {
	$rc = $1;
    }
    return $rc;
}

########################################################################
# Get the job state from a Job Status reply
########################################################################
sub GetState($)
{
    my $result = $_[0];
    my $rc;

    if ($result =~ /<state>(.*?)<\/state>/s)
    {
	$rc = $1;
    }

    return $rc;
}

########################################################################
# Translate the XML state value to a printable string
########################################################################
sub GetPrintableState($)
{
    my $state = $_[0];
    my %strings = (
	 Invalid                        => "Invalid",
	 Running                        => "Running",
	 Completed                      => "Completed",
	 Scanning                       => "Scanning",
	 Cancelled                      => "Cancelled",
         Paused                         => "Paused",
         Scheduled                      => "Scheduled",
         Updating                       => "Updating VLDB",
         NameSelect                     => "Selecting new name for directory",
         Renaming                       => "Renaming directory",
         CreateJunc                     => "Creating junction",
         Cleanup                        => "Cleanup",
         ReplayingLog                   => "Retrying Skipped Files",
         RenameLogfile                  => "Renaming Log File",
         NewLogfile                     => "Creating New Log File",
         MoveTrustees                   => "Moving Trustees",
         NewEFL                         => "Creating New EFL Epoch",
         ReplayingEFL                   => "Moving Modified Files",
         Starting                       => "Starting",
         Cancelling                     => "Cancelling",
         Pausing                        => "Pausing",
         Suspending                     => "Suspending",
         RetryUpdating                  => "Retry Update",
	 RetryNameSelect                => "Retry Name Select",
         RetryRenaming                  => "Retry Rename",
         RetryCreateJunc                => "Retry Create Junction",
         RetryCleanup                   => "Retry Cleanup",
         RetryReplay                    => "Retry Replay Log",
         RetryRenameLogfile             => "Retry Rename Log File",
         RetryNewLogfile                => "Retry Create New Log File",
         RetryMoveTrustees              => "Retry Move Trustees",
         RetryNewEFL                    => "Retry Create New EFL Epoch",
         RetryReplayEFL                 => "Retry Move Modified Files",
         FilesSkipped                   => "Files Were Skipped",
         CleanupFailed                  => "Cleanup Failed",
         Failed                         => "Failed",
         FailedFileRead                 => "Failed",
         FailedFileRestore              => "Failed",
         FailedBeginBackup              => "Failed",
         FailedLogin                    => "Failed",
         FailedTargetVersion            => "Failed",
         FailedNoManagementContext      => "Failed",
         FailedNotSameManagementContext => "Failed",
         FailedLogFile                  => "Failed",
         Unknown                        => "Unknown"
    );

    return $strings{$state};
}

########################################################################
# Translate a numeric error code value to a printable string
########################################################################
sub GetErrorText($)
{
    my $state = $_[0];
    my %strings = (
	 -10001  => "Invalid Incarnation Number",
	 -10002  => "Invalid Parameters",
	 -10003  => "Request was Aborted",
	 -10004  => "Database is Corrupt",
	 -10005  => "Insufficient Rights",
         -10006  => "Transaction Not Found",
         -10007  => "Could Not Cancel",
         -10008  => "No More Entries",
         -10009  => "Entry Not Found",
         -10010  => "Entry Already Exists",
         -10011  => "Invalid Opcode",
         -10012  => "Operation Not Supported",
         -10013  => "Insufficient Resources",
         -10014  => "Maximum # Servers Exceeded",
         -10015  => "Service is Stopped",
         -10016  => "Service is Not Stopped",
	 -10017  => "GUID Already Set",
         -10018  => "GUID Not Set",
         -10019  => "Internal Error",
         -10020  => "Operation Not Found",
         -10021  => "Operation Cannot Be Paused",
         -10022  => "Operation is Not Paused",
         -10023  => "The SYS: Volume Cannot Be Moved",
	 -10024  => "A Conflicting Move or Split Request Exists",
         -10025  => "Invalid Split Path",
         -10026  => "No Management Context",
    );

    return $strings{$state};
}

########################################################################
# Get additional info associated with a state
########################################################################
sub GetExtraInfo($$)
{
    my $state = $_[0];
    my $xml = $_[1];
    my $result;

    if ($state eq Completed)
    {
	my $skippedCount = GetSkippedCount($xml);
	if ($skippedCount == 0)
	{
	    $result = "";
	}
	elsif ($skippedCount == 1)
	{
	    $result = "1 file was not moved";
	}
	else
	{
	    $result = "$skippedCount files were not moved";
	}
    }
    elsif ($state eq Scanning)
    {
	my $dataSetCount = GetDataSetCount($xml);
	$result = "Data sets found: $dataSetCount";
    }
    elsif ($state eq Paused)
    {
	$result = GetComment($xml);
    }
    elsif ($state eq ReplayingEFL)
    {
	my $retryCount = GetRetryCount($xml);
	$result = "Attempt #$retryCount";
    }
    elsif ($state eq CleanupFailed)
    {
	$result = "Some files could not be deleted - resume to retry, cancel to ignore";
    }
    elsif ($state eq FilesSkipped)
    {
	$result = "Some files could not be moved - resume to retry, cancel to ignore";
    }
    elsif ($state eq FailedFileRead)
    {
	$result = "Could not read a file from source volume";
    }
    elsif ($state eq FailedFileRestore)
    {
	$result = "Could not write a file to target volume";
    }
    elsif ($state eq FailedBeginBackup)
    {
	$result = "Could not begin backup in SMS";
    }
    elsif ($state eq FailedLogin)
    {
	$result = "Could not authenticate to SMS";
    }
    elsif ($state eq FailedNoManagementContext)
    {
	$result = "No DFS management context defined for local server";
    }
    elsif ($state eq FailedNotSameManagementContext)
    {
	$result = "Target server not in same DFS management context as local server";
    }
    elsif ($state eq FailedLogFile)
    {
	$result = "Log file error";
    }
    else
    {
	$result = "";
    }

    return $result;
}

########################################################################
# Get the operation (MOVE/SPLIT) from a job status reply
########################################################################
sub GetOperation($)
{
    my $result = $_[0];
    my $rc;

    if ($result =~ /<jobType>(.*?)<\/jobType>/s)
    {
	$rc = $1;
    }
    return $rc;
}

########################################################################
# Get the source path from a Job Status reply
########################################################################
sub GetPath($)
{
    my $result = $_[0];
    my $rc;

    if ($result =~ /<srcPath>(.*?)<\/srcPath>/s)
    {
	$rc = $1;
    }
    return $rc;
}

########################################################################
# Get the skipped file count from a Job Status reply
########################################################################
sub GetSkippedCount($)
{
    my $result = $_[0];
    my $rc;

    if ($result =~ /<skippedFileCount>(\d+)<\/skippedFileCount>/s)
    {
	$rc = $1;
    }
    else
    {
	$rc = 0;
    }
    return $rc;
}

########################################################################
# Get the total files found count from a Job Status reply
########################################################################
sub GetDataSetCount($)
{
    my $result = $_[0];
    my $rc;

    if ($result =~ /<totalFiles>(\d+)<\/totalFiles>/s)
    {
	$rc = $1;
    }
    else
    {
	$rc = 0;
    }
    return $rc;
}

########################################################################
# Get the pause comment from a Job Status reply
########################################################################
sub GetComment($)
{
    my $result = $_[0];
    my $rc;

    if ($result =~ /<comment>(.*?)<\/comment>/s)
    {
	$rc = $1;
    }
    return $rc;
}

########################################################################
# Get the number of times we have replayed the EFL from a Job Status
# reply
########################################################################
sub GetRetryCount($)
{
    my $result = $_[0];
    my $rc;

    if ($result =~ /<retryCount>(\d+)<\/retryCount>/s)
    {
	$rc = $1;
    }
    else
    {
	$rc = 0;
    }
    return $rc;
}

########################################################################
# Get an example status response.  Used for testing.
########################################################################
sub GetTestStatusReply()
{
    my $result;

    $result = "<nssReply><volMn><getJobStatus>".
	          "<job>".
		      "<id>31415926535</id>".
		      "<jobType>MOVE</jobType>".
		      "<srcPath>PI:</srcPath>".
		      "<state>Running</state>".
		      "<pctComplete>31</pctComplete>".
		      "<time>12452542346</time>".
		      "<totalFiles>314</totalFiles>".
		  "</job>".
	          "<job>".
		      "<id>31415926536</id>".
		      "<jobType>SPLIT</jobType>".
		      "<srcPath>VOL1:one/after/pi</srcPath>".
		      "<state>Scanning</state>".
		      "<pctComplete>0</pctComplete>".
		      "<time>12452542346</time>".
		      "<totalFiles>456</totalFiles>".
		  "</job>".
	          "<job>".
		      "<id>27182818284</id>".
		      "<jobType>MOVE</jobType>".
		      "<srcPath>EE:</srcPath>".
		      "<state>Paused</state>".
		      "<pctComplete>27</pctComplete>".
		      "<time>12452542346</time>".
		      "<totalFiles>314</totalFiles>".
		      "<comment>The pause that refreshes</comment>".
		  "</job>".
	          "<job>".
		      "<id>77777777777</id>".
		      "<jobType>MOVE</jobType>".
		      "<srcPath>VOL7:</srcPath>".
		      "<state>RetryMoveTrustees</state>".
		      "<pctComplete>99</pctComplete>".
		      "<time>12452542346</time>".
		      "<totalFiles>7777</totalFiles>".
		  "</job>".
	          "<job>".
		      "<id>88888888888</id>".
		      "<jobType>MOVE</jobType>".
		      "<srcPath>VOL8:</srcPath>".
		      "<state>ReplayingEFL</state>".
		      "<pctComplete>99</pctComplete>".
		      "<time>12452542346</time>".
		      "<totalFiles>7777</totalFiles>".
		      "<retryCount>8</retryCount>".
		  "</job>".
		  "</getJobStatus></volMn></nssReply>";
    return $result;
}

########################################################################
# Prompt for a user name and collect it
########################################################################
sub GetUserName()
{
    my $uname;

    print "User Name: ";
    chomp($uname = <STDIN>);

    return $uname;
}

########################################################################
# Prompt for a password and collect it without echoing it
########################################################################
sub GetPassword()
{
    my $pwd;

    print "Password: ";
    system "stty -echo";
    chomp($pwd = <STDIN>);
    print "\n";
    system "stty echo";

    return $pwd;
}

########################################################################
# Set the datastream for a file handle to the passed in value.
########################################################################
sub SetDataStream(*$)
{
    my $fh = $_[0];
    my $dataStream = $_[1];

    my $result;
    my $command;

    $command = "<virtualIO><datastream name=\"$dataStream\"/></virtualIO>";
    seek $fh, 0, 0;
    if (!syswrite($fh, $command, length($command)))
    {
        $result .= "Unable to send datastream command to NDS management.  ";
        seek $fh, 0, 0;
        if (sysread($fh, $error, 3000)) 
        {
            $result .= $error;
        }
        $result .= "\n";
    }
    return $result;
}

########################################################################
# Write a command to a file and get the result
########################################################################
sub WriteCommand(*$)
{
    my $fh = $_[0];
    my $command = $_[1];

    my $result;

#    print("command=$command\n");
    seek $fh, 0, 0;
    if (!syswrite($fh, $command, length($command)))
    {
        $result .= "Unable to send command to virtual file.  ";
        seek $fh, 0, 0;
        if (sysread($fh, $error, 3000)) 
        {
            $result .= $error;
        }
        $result .= "\n";
    }
    else
    {
        seek $fh, 0, 0;
        sysread($fh, $reply, 3000);
        $result .= $reply;
    }
#       print("result=$result\n");
    return $result;
}

